searchData={"items":[{"type":"module","title":"Routex.Attrs","doc":"Provides an interface to access and update Routex attributes.\n\n  Extensions can make use of `Routex.Attrs` values provided by Routex itself,\nRoutex backends and other extensions. As those values are attributes to a\nroute, extension B can use values attributed to a route by extension A.\n\n  * To make the availability of the attributes as predictable as possible, Routex\n  uses a flat structure.\n  * Extension developers are encouraged to put as much information into the attributes\n  as possible.\n  * Extensions should add any fallback/default they might use themselves to the\n  attributes.","ref":"Routex.Attrs.html"},{"type":"function","title":"Routex.Attrs.cleanup/1","doc":"Removes non private fields from attrs. Input is either an opts map or a `Route`.","ref":"Routex.Attrs.html#cleanup/1"},{"type":"function","title":"Routex.Attrs.get/3","doc":"Get the value assigned to `key` from `route`'s options or return `default`. When no\nkey is provided, the function returns all options.","ref":"Routex.Attrs.html#get/3"},{"type":"function","title":"Routex.Attrs.get!/3","doc":"Get the value assigned to `key` from `route`'s options or raise if `key` not found. Accepts an optional\ncustom `error_msg` as third argument","ref":"Routex.Attrs.html#get!/3"},{"type":"function","title":"Routex.Attrs.merge/2","doc":"Merge `value` into `route`'s options.","ref":"Routex.Attrs.html#merge/2"},{"type":"function","title":"Routex.Attrs.private?/1","doc":"Returns true when the provided key or attribute is private.","ref":"Routex.Attrs.html#private?/1"},{"type":"function","title":"Routex.Attrs.put/2","doc":"Assigns `value` to `route`'s options.","ref":"Routex.Attrs.html#put/2"},{"type":"function","title":"Routex.Attrs.put/3","doc":"Assigns `value` to `key` in `route`'s options.","ref":"Routex.Attrs.html#put/3"},{"type":"function","title":"Routex.Attrs.update/2","doc":"Updates the options of a `route` by applying given `function`.","ref":"Routex.Attrs.html#update/2"},{"type":"function","title":"Routex.Attrs.update/3","doc":"Updates the value assigned to `key` of a `route`'s options by applying given `function`.","ref":"Routex.Attrs.html#update/3"},{"type":"module","title":"Routex.Backend","doc":"> #### `use Routex.Backend` {: .info}\n> When use'd this module generates a Routext backend module and\n> a configuration struct using the `configure/2` callbacks of\n> the extensions provided in `opts`.\n>\n> See also: [Routex Extensions](EXTENSIONS.md).\n\n**Example**\n\n     iex> defmodule MyApp.RtxBackend do\n     ...>  use Routex.Backend,\n     ...>   extensions: [\n     ...>    Routex.Extension.VerifiedRoutes,\n     ...>    Routex.Extension.AttrGetters,\n     ...>   ],\n     ...>   extension_x_config: [key: \"value\"]\n     ...> end\n     iex> IO.inspect(%MyApp.RtxBackend{})\n     %MyApp.RtxBackend{\n       extension_x_config: [key: \"value\"],\n       extensions: [Routex.Extension.VerifiedRoutes, Routex.Extension.AttrGetters],\n       verified_sigil_routex: \"~l\",\n       verified_sigil_original: \"~o\"\n     }\n\nValues in the configuration can be overridden by providing an override map to the `:private` option of a scope or route.\n\n**Example**\n\n    live /products, MyApp.Web.ProductIndexLive, :index, private: %{rtx: %{overridden_key: value}}","ref":"Routex.Backend.html"},{"type":"type","title":"Routex.Backend.t/0","doc":"A Routex backend module","ref":"Routex.Backend.html#t:t/0"},{"type":"module","title":"Routex.Branching","doc":"Provides a set of functions to build branched variants of macro's","ref":"Routex.Branching.html"},{"type":"function","title":"Routex.Branching.branch_macro/7","doc":"Takes a list of match patterns and creates the AST for branching variants\nfor all arities of `macro` in `module`.\n\n\n** Args\n- match_binding: the argument for the case clause\n- patterns: the match patterns of the case clause\n- transformer: transforms a original arguments value\n\n** Example\n\nWe want to create a branching variant of the `url` macro in `Phoenix,.VerifiedRoutes` module. The original\nmacro generates code that simply prints the given path argument, but we want to it to write multiple clauses and\nprefix the given argument based on the clause.\n\n  defmacro url(path, opts \\ []) do -> quote do IO.puts(path) end\n\nGiven this code:\n\n  patterns = [\"en\", \"nl\"]\n  match_binding = var!(external_var)\n  arg_transformer = fn pattern, arg -> \"europe/\" <> pattern <> \"/\" <> arg end\n\n  branch_macro(patterns, match_binding, arg_transformer, OriginalModule, :url,\n\tas: :url,\n\torig: :url_original,\n\targ_pos: fn arity -> arity - 1 end)\n\nA new macro is build which outputs the AST of the original macro, wrapped in a case clause given transformed arguments.\n\n  defmacro url(path, opts \\ []) do\n\t quote do\n\t   case match_binding do\n\t\t\t \"en\" -> Original.Module.url(\"europe/en/\" <> url, opts)\n\t\t\t \"nl\" -> Original.Module.url(\"europe/nl/\" <> url, opts)\n\t\t end\n end\nend","ref":"Routex.Branching.html#branch_macro/7"},{"type":"function","title":"Routex.Branching.build_case/8","doc":"","ref":"Routex.Branching.html#build_case/8"},{"type":"function","title":"Routex.Branching.build_default/3","doc":"","ref":"Routex.Branching.html#build_default/3"},{"type":"module","title":"Routex.Dev","doc":"Provides functions to aid during development","ref":"Routex.Dev.html"},{"type":"function","title":"Routex.Dev.inspect_ast/2","doc":"`Macro.escape/1` and `IO.inspect/2` the given input. Options are\npassed through to `IO.inspect`. Returns the input.","ref":"Routex.Dev.html#inspect_ast/2"},{"type":"function","title":"Routex.Dev.print_ast/2","doc":"Helper function to inspect AST as formatted code. Returns the\ninput.\n\n**Example**\n\n    iex> ast = quote do: Map.put(my_map, :key, value)\n    iex> print_ast(ast)\n    Map.put(my_map, :key, value)\n    ...actual AST...","ref":"Routex.Dev.html#print_ast/2"},{"type":"behaviour","title":"Routex.Extension","doc":"Specification for composable Routex extensions.\n\nOptional callbacks:\n- configure\n- transform\n- post_transform\n- create_helpers\n\nSee also: [Routex Extensions](EXTENSIONS.md)","ref":"Routex.Extension.html"},{"type":"callback","title":"Routex.Extension.configure/2","doc":"The `configure/2` callback is called in the first stage with the options\nprovided to `Routex` and the name of the Routext backend. It is expected to\nreturn a new list of options.","ref":"Routex.Extension.html#c:configure/2"},{"type":"callback","title":"Routex.Extension.create_helpers/3","doc":"The `create_helpers/3` callback is called in the last stage with a list of\nroutes belonging to a Routext backend, the name of the Routext backend and\nthe current environment. It is expected to return Elixir AST.\n\nThe AST is included in `MyAppWeb.Router.RoutexHelpers`.","ref":"Routex.Extension.html#c:create_helpers/3"},{"type":"callback","title":"Routex.Extension.post_transform/3","doc":"The `post_transform/1` callback is called in the third stage with a list of\nroutes belonging to a Routext backend. It is expected to return a list of\nPhoenix.Router.Route structs almost identical to the input, only adding\n`Routex.Attrs` -for own usage- is allowed.","ref":"Routex.Extension.html#c:post_transform/3"},{"type":"callback","title":"Routex.Extension.transform/3","doc":"The `transform/3` callback is called in the second stage with a list of\nroutes belonging to a Routext backend, the name of the configuration model\nand the current environment. It is expected to return a list of\nPhoenix.Router.Route structs with flattened `Routex.Attrs`.","ref":"Routex.Extension.html#c:transform/3"},{"type":"type","title":"Routex.Extension.backend/0","doc":"","ref":"Routex.Extension.html#t:backend/0"},{"type":"type","title":"Routex.Extension.env/0","doc":"","ref":"Routex.Extension.html#t:env/0"},{"type":"type","title":"Routex.Extension.opts/0","doc":"","ref":"Routex.Extension.html#t:opts/0"},{"type":"type","title":"Routex.Extension.routes/0","doc":"","ref":"Routex.Extension.html#t:routes/0"},{"type":"module","title":"Routex.Matchable","doc":"Matchables are an essential part of Routex. They are used to match run time\nroutes with compile time routes.\n\nThis module provides functions to create Matchables, convert them to match\npattern AST as well as function heads AST and to check if the routing values\nof two Matchable records match.","ref":"Routex.Matchable.html"},{"type":"function","title":"Routex.Matchable.match?/2","doc":"Returns whether two Matchable records match on their route defining properties. The first argument\nsupports string interpolation syntax (e.g \":param\" and \"*\") forming wildcards.","ref":"Routex.Matchable.html#match?/2"},{"type":"function","title":"Example - Routex.Matchable.match?/2","doc":"iex> route_record = %Phoenix.Router.Route{path: \"/posts/:id\"} |> Routex.Matchable.new()\n   > matching_record = \"/posts/1/foo=bar#top\" |> Routex.Matchable.new()\n   > failing_record = \"/products/1/foo=bar#op\" |> Routex.Matchable.new()\n\niex> match?(route_record, matching_record)\ntrue\niex match?(route_record, failing_record)\nfalse","ref":"Routex.Matchable.html#match?/2-example"},{"type":"macro","title":"Routex.Matchable.matchable/1","doc":"","ref":"Routex.Matchable.html#matchable/1"},{"type":"macro","title":"Routex.Matchable.matchable/2","doc":"","ref":"Routex.Matchable.html#matchable/2"},{"type":"function","title":"Routex.Matchable.new/1","doc":"Converts a binary URL, `Phoenix.Router.Route` or (sigil) AST argument into a Matchable record.","ref":"Routex.Matchable.html#new/1"},{"type":"function","title":"Examples - Routex.Matchable.new/1","doc":"iex> path = \"/posts/1?foo=bar#top\"\n\t\t > route = %Phoenix.Router.Route{path: \"/posts/:id\"}\n\t\t > ast = {:<<>>, [], [\"/products/\", {:\"::\", [], [{{:., [], [Kernel, :to_string]}, [from_interpolation: true], [{:id, [], Elixir}]}, {:binary, [], Elixir}]}]}\n\n\tiex> path_match = Routex.Matchable.new(path)\n\t{:matchable, [nil], [\"posts\", \"1\"], \"foo=bar\", \"top\", false}\n\n\tiex> route_match = Routex.Matchable.new(route)\n\t{:matchable, [], [\"posts\", \":id\"], nil, nil, false}\n\n\tiex> ast_match = Routex.Matchable.new(ast)\n\t{:matchable, [], [\"posts\", {:\"::\", [], [{{:., [], [Kernel, :to_string]}, [from_interpolation: true], [{:id, [], Elixir}]}, {:binary, [], Elixir}]}], nil, nil, false}","ref":"Routex.Matchable.html#new/1-examples"},{"type":"function","title":"Routex.Matchable.placeholders_to_ast/2","doc":"","ref":"Routex.Matchable.html#placeholders_to_ast/2"},{"type":"function","title":"Routex.Matchable.to_ast_segments/1","doc":"","ref":"Routex.Matchable.html#to_ast_segments/1"},{"type":"function","title":"Routex.Matchable.to_binary/1","doc":"A non conflicting function mimicking `to_string/1`","ref":"Routex.Matchable.html#to_binary/1"},{"type":"function","title":"Routex.Matchable.to_func/4","doc":"Creates a function named `name` which the first argument matching\na Matchable record pattern. Other arguments can be given with either a\ncatch all or a pattern.\n\nThe Matchable pattern is bound to `pattern`","ref":"Routex.Matchable.html#to_func/4"},{"type":"function","title":"Example - Routex.Matchable.to_func/4","doc":"iex> \"/some/path\"\n      >  |> Matchable.new()\n      >  |> Matchable.to_func(:my_func, [pattern_arg: \"fixed\", :catchall_arg], quote(do: :ok))","ref":"Routex.Matchable.html#to_func/4-example"},{"type":"function","title":"Routex.Matchable.to_pattern/1","doc":"Returns a match pattern for given `Matchable` record or `Phoenix.Router.Route`. The pattern can be used either as\nfunction argument or in a function body. As the patterns bind values, they can be used to convert input from\none pattern to another.\n\n\n*Example*\n    iex> \"/original/:arg1/:arg2\" |> Routex.Matchable.new() |> Routex.Matchable.to_pattern()\n\t{:{}, [], [:matchable, {:hosts, [], Routex.Matchable}, [\"original\", {:arg1, [], Routex.Matchable}, {:arg2, [], Routex.Matchable}], {:query, [], Routex.Matchable}, {:fragment, [], Routex.Matchable}, false]}\n\n\tiex> \"/recomposed/:arg2/:arg1\" |> Routex.Matchable.new() |> Routex.Matchable.to_pattern()\n\t{:{}, [], [:matchable, {:hosts, [], Routex.Matchable}, [\"recomposed\", {:arg2, [], Routex.Matchable}, {:arg1, [], Routex.Matchable}], {:query, [], Routex.Matchable}, {:fragment, [], Routex.Matchable}, false]}\n\n\n\tiex> \"/original/segment_1/segment_2\" |> Routex.Matchable.new() |> Routex.Matchable.to_pattern()\n\t{:{}, [], [:matchable, {:hosts, [], Routex.Matchable}, [\"original\", \"segment_1\", \"segment_2\"], {:query, [], Routex.Matchable}, {:fragment, [], Routex.Matchable}, false]}","ref":"Routex.Matchable.html#to_pattern/1"},{"type":"module","title":"Routex.Processing","doc":"This module provides everything needed to process Phoenix routes. It executes\nthe `transform` callbacks from extensions to transform `Phoenix.Router.Route`\nstructs and `create_helpers` callbacks to create one unified Helper module.\n\n**Powerful but thin**\nAlthough Routext is able to influence the routes in Phoenix applications in profound\nways, the framework and it's extensions are a suprisingly lightweight piece\nof compile-time middleware. This is made possible by the way router modules\nare pre-processed by `Phoenix.Router` itself.\n\nPrior to compilation of a router module, Phoenix Router registers all routes\ndefined in the router module using the attribute `@phoenix_routes`. Each\nroute is at that stage a `Phoenix.Router.Route` struct.\n\nAny route enclosed in a `preprocess_using` block has received a `:private`\nfield in which Routex has put which Routext backend to use for that\nparticular route. By enumerating the routes, we can process each route using\nthe properties of this configuration and set struct values accordingly. This\nprocessing is nothing more than (re)mapping the Route structs' values.\n\nAfter the processing by Routex is finished, the `@phoenix_routes` attribute\nin the router is erased and re-populated with the list of mapped\nPhoenix.Router.Route structs.\n\nOnce the router module enters the compilation stage, Routex is already out of\nthe picture and route code generation is performed by Phoenix Router.","ref":"Routex.Processing.html"},{"type":"function","title":"Routex.Processing.__before_compile__/1","doc":"Callback executed before compilation of a `Phoenix Router`. This callback is added\nto the `@before_compile` callbacks by `Routex.Router`.","ref":"Routex.Processing.html#__before_compile__/1"},{"type":"function","title":"Routex.Processing.exec_when_defined/5","doc":"Checks if the `callback` is defined. When defined it executes\nthe `callback` and returns the result , otherwise returns `default`.","ref":"Routex.Processing.html#exec_when_defined/5"},{"type":"function","title":"Routex.Processing.execute_callbacks/1","doc":"The main function of this module. Receives as only argument the environment of a\nPhoenix router module.","ref":"Routex.Processing.html#execute_callbacks/1"},{"type":"type","title":"Routex.Processing.backend/0","doc":"","ref":"Routex.Processing.html#t:backend/0"},{"type":"type","title":"Routex.Processing.extension_module/0","doc":"","ref":"Routex.Processing.html#t:extension_module/0"},{"type":"type","title":"Routex.Processing.routes/0","doc":"","ref":"Routex.Processing.html#t:routes/0"},{"type":"module","title":"Routex.Route","doc":"Function for working with Routex augmented Phoenix Routes","ref":"Routex.Route.html"},{"type":"function","title":"Routex.Route.exprs/2","doc":"Compatibility wrapper around `Phoenix.Router.Route.exprs`","ref":"Routex.Route.html#exprs/2"},{"type":"function","title":"Routex.Route.get_nesting/2","doc":"Returns the nesting level of an (ancestor) route. By default\nthe parent. This can be adjusted by providing an negative depth offset.","ref":"Routex.Route.html#get_nesting/2"},{"type":"function","title":"Routex.Route.group_by_method_and_origin/1","doc":"Returns routes grouped by the combination of method and origin path","ref":"Routex.Route.html#group_by_method_and_origin/1"},{"type":"function","title":"Routex.Route.group_by_method_and_path/2","doc":"","ref":"Routex.Route.html#group_by_method_and_path/2"},{"type":"function","title":"Routex.Route.group_by_nesting/2","doc":"Returns routes grouped by nesting level of an (ancestor) route. By default\ngroups by parent. This can be adjusted by providing an negative depth offset","ref":"Routex.Route.html#group_by_nesting/2"},{"type":"module","title":"Routex.Router","doc":"Provides macro (callbacks) to alter route definition before\ncompilation.\n\n> #### `use Routex.Router` {: .info}\n>\n> When you `use Routex.Router`, the Routex.Router module will\n> plug `Routex.Processing` between the definition of routes and the\n> compilation of the router module. It also imports the `preprocess_using`\n> macro which can be used to mark routes for Routex preprocessing using the\n> Routex configuration/backend provided as first argument.","ref":"Routex.Router.html"},{"type":"macro","title":"Routex.Router.preprocess_using/3","doc":"Wraps each enclosed route in a scope, marking it for processing by Routex\nusing given `backend`. `opts` can be used to partially override the given\nconfiguration.\n\nReplaces interpolation syntax with a string for macro free processing by\nextensions. Format: `[rtx.{binding}]`.","ref":"Routex.Router.html#preprocess_using/3"},{"type":"module","title":"Routex.Utils","doc":"Provides an interface to functions which can be used in extensions.","ref":"Routex.Utils.html"},{"type":"function","title":"Routex.Utils.get_branch/1","doc":"","ref":"Routex.Utils.html#get_branch/1"},{"type":"function","title":"Routex.Utils.get_helper_ast/1","doc":"Returns the AST to get the current branch from process dict or from  assigns, conn or socket\nbased on the available variables in the __CALLER__ module.","ref":"Routex.Utils.html#get_helper_ast/1"},{"type":"function","title":"Routex.Utils.print/1","doc":"Prints an indented text. Should be used when printing messages in\nthe terminal during compile time.","ref":"Routex.Utils.html#print/1"},{"type":"module","title":"Routex.Extension.AlternativeGetters","doc":"Creates helper functions to get a list of alternative slugs and their Routex\nattributes by providing the function a binary url.","ref":"Routex.Extension.AlternativeGetters.html"},{"type":"module","title":"Configuration - Routex.Extension.AlternativeGetters","doc":"```diff\n# file /lib/example_web/routex_backend.ex\ndefmodule ExampleWeb.RoutexBackend do\n  use Routex.Backend,\n  extensions: [\n    Routex.Extension.Alternatives,\n+   Routex.Extension.AlternativeGetters,\n    Routex.Extension.AttrGetters\n],\n```\n\n## `Routex.Attrs`\n**Requires**\n- none\n\n**Sets**\n- none","ref":"Routex.Extension.AlternativeGetters.html#module-configuration"},{"type":"module","title":"Helpers - Routex.Extension.AlternativeGetters","doc":"- alternatives(url :: String.t()) :: struct()\n\n**Example**\n```elixir\niex> ExampleWeb.Router.RoutexHelpers.alternatives(\"/products/12?foo=baz\")\n[ %Routex.Extension.AlternativeGetters{\n  slug: \"products/12/?foo=baz\",\n  match?: true,\n  attrs: %{\n    __branch__: [0, 12, 0],\n    __origin__: \"/products/:id\",\n    [...attributes set by other extensions...]\n  }},\n  %Routex.Extension.AlternativeGetters{\n  slug: \"/europe/products/12/?foo=baz\",\n  match?: true,\n  attrs: %{\n    __branch__: [0, 12, 1],\n    __origin__: \"/products/:id\",\n    [...attributes set by other extensions...]\n  }},\n %Routex.Extension.AlternativeGetters{\n  slug: \"/asia/products/12/?foo=baz\",\n  match?: true,\n  attrs: %{\n    __branch__: [0, 12, 1],\n    __origin__: \"/products/:id\",\n    [...attributes set by other extensions...]\n  }},\n]\n```","ref":"Routex.Extension.AlternativeGetters.html#module-helpers"},{"type":"module","title":"Routex.Extension.Alternatives","doc":"Creates alternative routes based on `branches` configured in a Routex backend\nmodule. Branches can be nested and each branch can provide `Routex.Attrs` to be shared\nwith other extensions.","ref":"Routex.Extension.Alternatives.html"},{"type":"module","title":"Configuration - Routex.Extension.Alternatives","doc":"```diff\n# file /lib/example_web/routex_backend.ex\n# This example uses a `Struct` for custom attributes, so there is no attribute inheritance;\n# only struct defaults. When using maps, nested branches will inherit attributes from their parent.\n\n+ defmodule ExampleWeb.RoutexBackend.AltAttrs do\n+  @moduledoc false\n+  defstruct [:contact, locale: \"en\"]\n+ end\n\ndefmodule ExampleWeb.RoutexBackend do\n+ alias ExampleWeb.RoutexBackend.AltAttrs\n\nuse Routex.Backend,\nextensions: [\n+ Routex.Extension.Alternatives,\nRoutex.Extension.AttrGetters\n],\n+ alternatives: %{\n+    \"/\" => %{\n+      attrs: %AltAttrs{contact: \"root@example.com\"},\n+      branches: %{\n+        \"/europe\" => %{\n+          attrs: %AltAttrs{contact: \"europe@example.com\"},\n+          branches: %{\n+            \"/nl\" => %{attrs: %AltAttrs{locale: \"nl\", contact: \"verkoop@example.nl\"}},\n+            \"/be\" => %{attrs: %AltAttrs{locale: \"nl\", contact: \"handel@example.be\"}}\n+          }\n+        },\n+      \"/gb\" => %{attrs: %AltAttrs{contact: \"sales@example.com\"}\n+    }\n+  },\n+ alternatives_prefix: false  # whether to automatically prefix routes, defaults to true\n```","ref":"Routex.Extension.Alternatives.html#module-configuration"},{"type":"module","title":"Pseudo result - Routex.Extension.Alternatives","doc":"⇒ /products/:id/edit              locale: \"en\", contact: \"rootexample.com\"\n    /products/:id/edit  ⇒ /europe/nl/products/:id/edit    locale: \"nl\", contact: \"verkoop@example.nl\"\n                        ⇒ /europe/be/products/:id/edit    locale: \"nl\", contact: \"handel@example.be\"\n                        ⇒ /gb/products/:id/edit           locale: \"en\", contact: \"sales@example.com\"\n\n## `Routex.Attrs`\n**Requires**\n- none\n\n**Sets**\n- **any key/value in `:attrs`**\n- branch_helper\n- branch_alias\n- branch_prefix\n- branch_opts\n- alternatives (list of `Phoenix.Route.Route`)","ref":"Routex.Extension.Alternatives.html#module-pseudo-result"},{"type":"module","title":"Routex.Extension.Assigns","doc":"Extracts `Routex.Attrs` from the route and makes them available in components\nand controllers with the `@` assigns operator (optionally under a namespace).","ref":"Routex.Extension.Assigns.html"},{"type":"module","title":"Options - Routex.Extension.Assigns","doc":"- `namespace`: when set creates a named collection of `Routex.Attrs`\n- `attrs`: when set defines keys of `Routex.Attrs` to make available","ref":"Routex.Extension.Assigns.html#module-options"},{"type":"module","title":"Configuration - Routex.Extension.Assigns","doc":"```diff\n# file /lib/example_web/routex_backend.ex\ndefmodule ExampleWeb.RoutexBackend do\n  use Routex.Backend,\n  extensions: [\n+   Routex.Extension.Assigns,\nRoutex.Extension.AttrGetters\n],\n+ assigns: %{namespace: :rtx, attrs: [:branch_helper, :locale, :contact, :name]}\n```","ref":"Routex.Extension.Assigns.html#module-configuration"},{"type":"module","title":"Pseudo result - Routex.Extension.Assigns","doc":"# in (h)eex template\n    @rtx.branch_helper   ⇒  \"eu_nl\"\n    @rtx.locale         ⇒  \"nl\"\n    @rtx.contact        ⇒  \"verkoop@example.nl\"\n    @rtx.name           ⇒  \"The Netherlands\"\n\n## `Routex.Attrs`\n**Requires**\n- none\n\n**Sets**\n- assigns","ref":"Routex.Extension.Assigns.html#module-pseudo-result"},{"type":"module","title":"Example use case - Routex.Extension.Assigns","doc":"Combine with `Routext.Extension.Alternatives` to make compile time, branch\nbound assigns available to components and controllers.","ref":"Routex.Extension.Assigns.html#module-example-use-case"},{"type":"module","title":"Routex.Extension.AttrGetters","doc":"Creates helper functions to get the `Routex.Attrs` given a binary url or a\nlist of path segments. This way the attributes for route can be lazily\nloaded.","ref":"Routex.Extension.AttrGetters.html"},{"type":"module","title":"Configuration - Routex.Extension.AttrGetters","doc":"```diff\n# file /lib/example_web/routex_backend.ex\ndefmodule ExampleWeb.RoutexBackend do\n  use Routex.Backend,\n  extensions: [\n+   Routex.Extension.AttrGetters,\n],\n```\n\n## `Routex.Attrs`\n**Requires**\n- none\n\n**Sets**\n- none","ref":"Routex.Extension.AttrGetters.html#module-configuration"},{"type":"module","title":"Helpers - Routex.Extension.AttrGetters","doc":"- attrs(url :: binary) :: map()\n- attrs(segments :: list) :: map()\n\n**Example**\n```elixir\niex> ExampleWeb.Router.RoutexHelpers.attrs(\"/europe/nl/producten/?foo=baz\")\n%{\n  __branch__: [0, 9, 3],\n  __origin__: \"/products\",\n  backend: ExampleWeb.LocalizedRoutes,\n  contact: \"verkoop@example.nl\",\n  locale: \"nl\",\n  branch_name: \"The Netherlands\",\n  branch_helper: \"europe_nl\",\n}\n```","ref":"Routex.Extension.AttrGetters.html#module-helpers"},{"type":"module","title":"Routex.Extension.Cldr","doc":"Adapter for projects using :ex_cldr. It generates the configuration\nfor `Routex.Extension.Alternatives`.","ref":"Routex.Extension.Cldr.html"},{"type":"module","title":"Interpolating Locale Data - Routex.Extension.Cldr","doc":"Interpolation is provided by `Routex.Extension.`Interpolation`, which\nis able to use any `Routex.Attr` for interpolation. See it's documentation\nfor additional options.\n\nWhen using this Cldr extension, the following interpolations are supported as they\nare set as `Routex.Attr`:\n\n* `locale` will interpolate the Cldr locale name\n* `locale_display` will interpolate the Cldr locale display name\n* `language` will interpolate the Cldr language name\n* `territory` will interpolate the Cldr territory code\n\nSome examples are:\n```elixir\npreprocess_using ExampleWeb.RoutexBackend do\n  scope \"/#{territory}/territory/\" do\n    get \"/#{locale}/locale/pages/:page\", PageController, :show\n    get \"/language/#{language}/pages/:page\", PageController, :show\n  end\nend\n```","ref":"Routex.Extension.Cldr.html#module-interpolating-locale-data"},{"type":"module","title":"Configuration - Routex.Extension.Cldr","doc":"```diff\ndefmodule ExampleWeb.RoutexBackend do\nuse Routex.Backend,\nextensions: [\n+ Routex.Extension.Cldr,\n+ Routex.Extension.Alternatives,\n+ Routex.Extension.Interpolation, #  when using routes with interpolation\n+ Routex.Extension.Translations,  # when using translated routes\n+ Routex.Extension.VerifiedRoutes,\n  [...]\n  Routex.Extension.AttrGetters\n],\n+ cldr_backend: MyApp.Cldr,\n+ translations_backend: MyApp.Gettext,  #  when using translated routes\n+ translations_domain: \"routes\",  #  when using translated routes\n+ alternatives_prefix: false,  #  when using routes with interpolation\n+ verified_sigil_routex: \"~q\", #  consider using ~p, see `Routex.Extension.VerifiedRoutes`\n```\n\n```diff\ndefmodule ExampleWeb.Router\n# require your Cldr backend module before `use`ing the router.\n+ require ExampleWeb.Cldr\n\nuse ExampleWeb, :router\n\nimport ExampleWeb.UserAuth\n```","ref":"Routex.Extension.Cldr.html#module-configuration"},{"type":"module","title":"Pseudo result - Routex.Extension.Cldr","doc":"This extension injects `:alternatives` into your configuration.\n See the documentation of `Routex.Extension.Alternatives` to see\n more options and the pseudo result.\n\n```\n alternatives: %{\n  \"/\" => %{\n    attrs: %{\n      language: \"en\",\n      locale: \"en\",\n      territory: \"US\",\n      locale_display: \"English (United States)\"\n    },\n    branches: %{\n      \"/en\" => %{\n        language: \"en\",\n        locale: \"en\",\n        territory: \"US\",\n        locale_dispay: \"English\"\n      },\n      \"/fr\" => %{\n        language: \"fr\",\n        locale: \"fr\",\n        territory: \"FR\",\n        locale_display: \"français\"\n      }\n   }\n }\n\n```\n\n## `Routex.Attrs`\n**Requires**\n- none\n\n**Sets**\n- language\n- locale\n- locale_display\n- territory","ref":"Routex.Extension.Cldr.html#module-pseudo-result"},{"type":"function","title":"Routex.Extension.Cldr.configure/2","doc":"","ref":"Routex.Extension.Cldr.html#configure/2"},{"type":"function","title":"Routex.Extension.Cldr.display_name/2","doc":"","ref":"Routex.Extension.Cldr.html#display_name/2"},{"type":"module","title":"Routex.Extension.Cloak","doc":"Transforms routes to be unrecognizable.\n\n\n> #### Warning {: .warning}\n>\n> This extension is intended for testing and demonstration. It's\n> behavior may change over time.","ref":"Routex.Extension.Cloak.html"},{"type":"module","title":"Configuration - Routex.Extension.Cloak","doc":"```diff\n# file /lib/example_web/routex_backend.ex\ndefmodule ExampleWeb.RoutexBackend do\n  use Routex.Backend,\n  extensions: [\n+  Routex.Extension.Cloak,\n   Routex.Extension.AttrGetters\n],\n```","ref":"Routex.Extension.Cloak.html#module-configuration"},{"type":"module","title":"Pseudo result - Routex.Extension.Cloak","doc":"/products/  ⇒ /1\n    /products/:id/edit  ⇒ rewrite: /:id/02      ⇒ in browser: /1/02, /2/02/ etc...\n    /products/:id/show/edit  ⇒ rewrite: /:id/03   ⇒ in browser: /1/03, /2/03/ etc...\n\n## `Routex.Attrs`\n**Requires**\n- none\n\n**Sets**\n- none","ref":"Routex.Extension.Cloak.html#module-pseudo-result"},{"type":"module","title":"Routex.Extension.Interpolation","doc":"A route may be defined with a routes `Routex.Attrs` interpolated\ninto it. These interpolations are specified using the usual `#{variable}`\ninterpolation syntax.\n\n> #### In combination with... {: .info}\n> This plugin makes a good duo with `Routex.Extension.Alternatives`. You might\n> want to disable auto prefixing for the whole Routex backend (see\n> `Routex.Extension.Alternatives`) or per route (see `Routex`for\n> instructions).\n\n> #### Bare base route {: .warn}\n> The route as specified in the Router will be stripped from any\n> interpolation syntax. This allows you to still use routes without interpolation\n> syntax in your templates, have them verified by Verified Routes while generating\n> routes with interpolated attributes at run time.","ref":"Routex.Extension.Interpolation.html"},{"type":"module","title":"Usage - Routex.Extension.Interpolation","doc":"```diff\n# file /lib/example_web/routes.ex\nlive \"/products/#{locale}/:id\", ProductLive.Index, :index\n```","ref":"Routex.Extension.Interpolation.html#module-usage"},{"type":"module","title":"Pseudo result - Routex.Extension.Interpolation","doc":"# in combination with Routex.Extension.Alternatives with auto prefix\n    # disabled and 3 branches. It splits the routes and sets the :locale\n    # attribute which is used for interpolation.\n\n                   ⇒ /products/en/:id\n    /products/:id/ ⇒ /products/fr/:id\n                   ⇒ /products/fr/:id\n\n## `Routex.Attrs`\n**Requires**\n- none\n\n**Sets**\n- none","ref":"Routex.Extension.Interpolation.html#module-pseudo-result"},{"type":"module","title":"Routex.Extension.RouteHelpers","doc":"Provides route helpers with support for automatic selecting alternatives\nroutes. The helpers can be used to override Phoenix' defaults as they are\na drop-in replacements.\n\nOnly use this extension when you make use of extensions generating alternative\nroutes, as otherwise the result will be the same as the official helpers.","ref":"Routex.Extension.RouteHelpers.html"},{"type":"module","title":"Configuration - Routex.Extension.RouteHelpers","doc":"```diff\n# file /lib/example_web/routex_backend.ex\ndefmodule ExampleWeb.RoutexBackend do\n  use Routex.Backend,\n  extensions: [\n+   Routex.Extension.RouteHelpers,\n    Routex.Extension.AttrGetters\n],\n```\n\nPhoenix < 1.7 created an alias `Routes` by default. You can either replace it\nor add an alias for RoutexHelpers. Phoenix >= 1.7 deprecated the helpers\nin favor of Verified Routes.\n\nIn the example below we 'override' the default `Routes` alias to use\nRoutex' Route Helpers as a drop-in replacement, but keep the original helpers\nfunctions available by using alias `OriginalRoutes`.\n\n\n```diff\n# file /lib/example_web.ex\ndefp routex_helpers do\n\n+ alias ExampleWeb.Router.Helpers, as: OriginalRoutes\n+ alias ExampleWeb.Router.RoutexHelpers, as: Routes\n\nend\n```","ref":"Routex.Extension.RouteHelpers.html#module-configuration"},{"type":"module","title":"Pseudo result - Routex.Extension.RouteHelpers","doc":"# When alternatives are created it uses auto-selection to keep the user 'in branch'.\n\n    # in (h)eex template\n     Product #1 \n\n    # is replaced during during compilation with:\n    case alternative do\n       nil ⇒  \"/products/#{product}\"\n      \"en\" ⇒  \"/products/#{product}\"\n      \"nl\" ⇒  \"/europe/nl/products/#{product}\"\n      \"be\" ⇒  \"/europe/be/products/#{product}\"\n    end\n\n## `Routex.Attrs`\n**Requires**\n- none\n\n**Sets**\n- none","ref":"Routex.Extension.RouteHelpers.html#module-pseudo-result"},{"type":"function","title":"Routex.Extension.RouteHelpers.build_case/1","doc":"","ref":"Routex.Extension.RouteHelpers.html#build_case/1"},{"type":"function","title":"Routex.Extension.RouteHelpers.build_case_clauses/4","doc":"","ref":"Routex.Extension.RouteHelpers.html#build_case_clauses/4"},{"type":"function","title":"Routex.Extension.RouteHelpers.dynamic_delegate_with_arity/3","doc":"","ref":"Routex.Extension.RouteHelpers.html#dynamic_delegate_with_arity/3"},{"type":"function","title":"Routex.Extension.RouteHelpers.dynamic_fn_with_arity/3","doc":"","ref":"Routex.Extension.RouteHelpers.html#dynamic_fn_with_arity/3"},{"type":"module","title":"Routex.Extension.Translations","doc":"Enables users to enter URLs using localized terms which can enhance user engagement\nand content relevance.\n\nExtracts segments of a routes' path to a translations domain file (default: `routes.po`)\nfor translation. At compile-time it combines the translated segments to transform routes.\n\nThis extension expects either a `:language` attribute or a `:locale` attribute. When only\n`:locale` is provided it will try to extract the language from the locale tag. This algorithm is\ncovers Alpha-2 and Alpha-3 codes (see:\n[ISO](https://datatracker.ietf.org/doc/html/rfc5646#section-2.2.1))","ref":"Routex.Extension.Translations.html"},{"type":"module","title":"Configuration - Routex.Extension.Translations","doc":"```diff\ndefmodule ExampleWeb.RoutexBackend do\nuse Routex.Backend,\nextensions: [\n+ Routex.Extension.Translations,\n  Routex.Extension.AttrGetters\n]\n+ translations_backend: MyApp.Gettext,\n+ translations_domain: \"routes\",\n```","ref":"Routex.Extension.Translations.html#module-configuration"},{"type":"module","title":"Pseudo result - Routex.Extension.Translations","doc":"# when translated to Spanish in the .po file\n    # - products: producto\n    # - edit: editar\n\n    /products/:id/edit  ⇒ /producto/:id/editar\n\n## `Routex.Attrs`\n**Requires**\n- language || locale\n\n**Sets**\n- none","ref":"Routex.Extension.Translations.html#module-pseudo-result"},{"type":"module","title":"Use case(s) - Routex.Extension.Translations","doc":"This extension can be combined with `Routext.Extension.Alternatives` to create\nmultilingual routes.\n\nUse Alternatives to create new branches and provide a `:language` or `:locale` per branch and\nTranslations to translate the alternative routes.\n\n                        ⇒ /products/:id/edit                  language: \"en\"\n    /products/:id/edit  ⇒ /nederland/producten/:id/bewerken   language: \"nl\"\n                        ⇒ /espana/producto/:id/editar         language: \"es\"","ref":"Routex.Extension.Translations.html#module-use-case-s"},{"type":"module","title":"Routex.Extension.VerifiedRoutes","doc":"Supports the use of unmodified route paths in controllers and templates while using transformed\nand/or branching paths with compile-time verification and dynamic runtime behavior.\n\n> #### Implementation summary {:.info}\n> Each sigil and function eventualy delegates to the official\n> `Phoenix.VerifiedRoutes`.  If a non-branching route is provided it will\n> simply delegate to the official Phoenix function. If a branching route is\n> provided, it will use a branching mechanism before delegation.","ref":"Routex.Extension.VerifiedRoutes.html"},{"type":"module","title":"Alternative Verified Route sigil - Routex.Extension.VerifiedRoutes","doc":"Provides a sigil (default: `~l`) to verify transformed and/or branching routes.\nThe sigil to use can be set to `~p` to override the default of Phoenix as\nit is a drop-in replacement. If you choose to override the default Phoenix sigil,\nit is renamed (default: `~o`) and can be used when unaltered behavior is required.","ref":"Routex.Extension.VerifiedRoutes.html#module-alternative-verified-route-sigil"},{"type":"module","title":"Variants of url/{2,3,4} and path/{2,3} - Routex.Extension.VerifiedRoutes","doc":"Provides branching variants of (and delegates to) macro's provided by\n`Phoenix.VerifiedRoutes`. Both new macro's detect whether branching should be\napplied.","ref":"Routex.Extension.VerifiedRoutes.html#module-variants-of-url-2-3-4-and-path-2-3"},{"type":"module","title":"Options - Routex.Extension.VerifiedRoutes","doc":"- `verified_sigil_routex`: Sigil to use for Routex verified routes (default `\"~l\"`)\n- `verified_sigil_phoenix`: Replacement for the native (original) sigil when `verified_sigil_routex`\n  is set to \"~p\". (default: `\"~o\"`)\n - `verified_url_routex`: Function name to use for Routex verified routes powered `url`. (default: `:rtx_url`)\n- `verified_url_phoenix`: Replacement for the native `url` function when `verified_url_routex`\n  is set to `:url`. (default: `:phx_url`)\n - `verified_path_routex`: Function name to use for Routex verified routes powered `path` (default `:rtx_path`)\n- `verified_path_phoenix`: Replacement for the native `path` function  when `verified_path_routex`\n  is set to `:path`. (default: `:phx_path`)\n\nWhen `verified_sigil_routex` is set to \"~p\" an additional change must be made.\n\n```diff\n# file /lib/example_web.ex\ndefp routex_helpers do\n+ import Phoenix.VerifiedRoutes, only: :functions\n  import ExampleWeb.Router.Routex\nend\n```","ref":"Routex.Extension.VerifiedRoutes.html#module-options"},{"type":"module","title":"Configuration - Routex.Extension.VerifiedRoutes","doc":"```diff\n# file /lib/example_web/routex_backend.ex\ndefmodule ExampleWeb.RoutexBackend do\n  use Routex.Backend,\n  extensions: [\n    Routex.Extension.Alternatives,\n    [...]\n+   Routex.Extension.VerifiedRoutes,\n    Routex.Extension.AttrGetters\n],\n+ verified_sigil_routex: \"~p\",\n+ verified_sigil_phoenix: \"~o\",\n+ verified_url_routex: :url,\n+ verified_url_phoenix: :url_native,\n+ verified_path_routex: :path,\n+ verified_path_phoenix: :path_native,\n```","ref":"Routex.Extension.VerifiedRoutes.html#module-configuration"},{"type":"module","title":"Pseudo result (simplified) - Routex.Extension.VerifiedRoutes","doc":"# given Routex is configured to use ~l\n    # given Phoenix is assigned ~o (for example clarity)\n\n    # given other extensions have caused a route transformation\n    ~o\"/products/#{product}\"   ⇒  ~p\"/products/#{products}\"\n    ~l\"/products/#{product}\"   ⇒  ~p\"/transformed/products/#{product}\"\n\n    # given another extension has generated branches / alternative routes\n    ~o\"/products/#{product}\"  ⇒  ~p\"/products/#{products}\"\n    ~l\"/products/#{product}\"  ⇒\n            case branch do\n              nil ⇒  ~p\"/products/#{product}\"\n              \"en\" ⇒  ~p\"/products/en/#{product}\"\n              \"eu_nl\" ⇒  ~p\"/europe/nl/products/#{product}\"\n              \"eu_be\" ⇒  ~p\"/europe/be/products/#{product}\"\n            end\n\n## `Routex.Attrs`\n**Requires**\n- none\n\n**Sets**\n- none","ref":"Routex.Extension.VerifiedRoutes.html#module-pseudo-result-simplified"},{"type":"exception","title":"Routex.Extension.Interpolation.NonUniqError","doc":"Raised when a list of routes contains routes with the same path and verb.\n\n```elixir\n[%Route{\n  path: \"/foo\"\n  verb: :get},\n%Route{\n  path: \"/foo\"\n  verb: :post}, # <-- different\n%Route{\n  path: \"/foo\"\n  verb: :get} # <-- duplicate\n]\n```\n\nSolution: use a combination of interpolated attributes that form a unique set.","ref":"Routex.Extension.Interpolation.NonUniqError.html"},{"type":"module","title":"Routex.Extension.Alternatives.Branch.Flat","doc":"Struct for flattened branch","ref":"Routex.Extension.Alternatives.Branch.Flat.html"},{"type":"type","title":"Routex.Extension.Alternatives.Branch.Flat.t/0","doc":"","ref":"Routex.Extension.Alternatives.Branch.Flat.html#t:t/0"},{"type":"module","title":"Routex.Extension.Alternatives.Branch.Nested","doc":"Struct for branch with optionally nested branches","ref":"Routex.Extension.Alternatives.Branch.Nested.html"},{"type":"type","title":"Routex.Extension.Alternatives.Branch.Nested.t/0","doc":"","ref":"Routex.Extension.Alternatives.Branch.Nested.html#t:t/0"},{"type":"module","title":"Routex.Extension.Alternatives.Config","doc":"Module to create and validate a Config struct","ref":"Routex.Extension.Alternatives.Config.html"},{"type":"type","title":"Routex.Extension.Alternatives.Config.t/0","doc":"","ref":"Routex.Extension.Alternatives.Config.html#t:t/0"},{"type":"module","title":"Routex.Extension.Alternatives.Exceptions","doc":"","ref":"Routex.Extension.Alternatives.Exceptions.html"},{"type":"exception","title":"Routex.Extension.Alternatives.Exceptions.AttrsMismatchError","doc":"Raised when the custom attributes of branches do not have the same keys.\n\n```elixir\n%{\n  branches: %{\n    \"/\"      => %{attrs: %{key1: 1, key2: 2}},\n    \"/other\" => %{attrs: %{key1: 1}} # missing :key2\n  }\n}\n```\n\nTo fix this, make the attribute maps consistent or use an attributes struct.","ref":"Routex.Extension.Alternatives.Exceptions.AttrsMismatchError.html"},{"type":"exception","title":"Routex.Extension.Alternatives.Exceptions.MissingRootSlugError","doc":"Raised when the branch map does not start with the root branch \"/\".\n\n```elixir\n%{\n  branches: %{\n    \"/first\"  =>    %{attrs: %{key1: 1}},\n    \"/other\"  =>    %{attrs: %{key1: 1}}},\n}\n```\n\nTo fix this, include a branch for the root \"/\".\n\n```elixir\n`%{\n  branches: %{\n    \"/\" => %{\n      attrs: %{level: 1}\n      branches: %{\n        \"/first\"  =>    %{attrs: %{level: 2}},\n        \"/other\"  =>    %{attrs: %{level: 2}}\n      }\n    },\n  }\n}\n```","ref":"Routex.Extension.Alternatives.Exceptions.MissingRootSlugError.html"},{"type":"extras","title":"README","doc":"![Coveralls](https://img.shields.io/coveralls/github/BartOtten/routex)\n[![Build Status](https://github.com/BartOtten/routex/actions/workflows/elixir.yml/badge.svg?event=push)](https://github.com/BartOtten/routex/actions/workflows/elixir.yml)\n[![Last Updated](https://img.shields.io/github/last-commit/BartOtten/routex.svg)](https://github.com/BartOtten/routex/commits/main)\n[![Hex.pm](https://img.shields.io/hexpm/v/routex)](https://hex.pm/packages/routex)\n![Hex.pm](https://img.shields.io/hexpm/l/routex)\n\n\n# Routex: Supercharge your Phoenix Router\n\nRoutex is an feature rich routing framework build on top of Phoenix Router. It\nprovides additional routing features for common use cases. Situated between\nroute definition and compilation, Routex has zero to none impact on run time\nperformance.","ref":"readme.html"},{"type":"extras","title":"Top Features and Benefits - README","doc":"- **extension driven**: mix and match the features you need.\n- **performant**: (close to) zero run time impact.\n- **extensible**: write your own routing features with ease.\n- **plug-and-play**: enable site wide features with a single config change.\n- **easy setup**: test drive Routex with minimal changes to the codebase.","ref":"readme.html#top-features-and-benefits"},{"type":"extras","title":"Documentation - README","doc":"[HexDocs](https://hexdocs.pm/routex) (stable) and [GitHub\nPages](https://bartotten.github.io/routex) (development).\n\nSummaries of [official Routex extensions](#extensions)","ref":"readme.html#documentation"},{"type":"extras","title":"Requirements and Installation - README","doc":"See the [Usage Guide](USAGE.md) for the requirements and installation\ninstructions.","ref":"readme.html#requirements-and-installation"},{"type":"extras","title":"Example - README","doc":"Localize your Phoenix website with multilingual URLs and custom template\nassigns; enhancing user engagement and content relevance. This example combines\na few of the included extensions: `Alternatives`, `Translations`, `Assigns` and\n`Verified Routes`.\n\n                        ⇒ /products/:id/edit                      @loc.locale = \"en_US\"\n    /products/:id/edit  ⇒ /eu/nederland/producten/:id/bewerken    @loc.locale = \"nl_NL\"\n                        ⇒ /eu/espana/producto/:id/editar          @loc.locale = \"es_ES\"\n                        ⇒ /gb/products/:id/edit                   @loc.locale = \"en_GB\"\n\n- `Alternatives` to generate 4 route branches with custom attributes per branch.\n- `Translation` to localize the urls enhancing user engagement and content relevance.\n- `Assigns` to access attributes of a route in LiveViews, components and\n  controllers using `loc` as namespace (`@loc.locale`)\n- `Verified Routes` to keep  `~p\"/products/#{product}/edit\"` in templates. At run-time the route is\n  combined with the set `locale` to pick the localized alternative route.","ref":"readme.html#example"},{"type":"extras","title":"Routex vs Cldr Routes vs Phoenix Localized Routes - README","doc":"The capabilities and advancements within `Routex` surpass those of `Phoenix\nLocalized Routes`, offering a comprehensive array of features. As Phoenix\nLocalized Routes has stagnated in its development, developers are strongly\nadvised to transition to Routex for a more robust solution.\n\nWhen considering `Routex` against `Cldr Routes`, it's akin to comparing Apple to\nLinux. Cldr Routes is a limited walled garden but is developed by the main Cldr\ndeveloper ensuring maximum compatibility. Routex on the other hand boasts a\nwider and more dynamic feature scope providing maximum freedom. Its primary\nadvantages over Cldr Routes are it's extension mechanism and the minimized\nnecessity for code modifications throughout a codebase.\n\nBut why choose when you can have Cldr through [the Cldr extension for\nRoutex](`Routex.Extension.Cldr`)?","ref":"readme.html#routex-vs-cldr-routes-vs-phoenix-localized-routes"},{"type":"extras","title":"Comparison table - README","doc":"| Feature             | Routex     | Cldr Routes | PLR        |\n|---------------------|------------|-------------|------------|\n| Route encapsulation | Full  [^1] | Limited     | Limited    |\n| Route manipulation  | Full  [^2] | Limited     | Limited    |\n| Route interpolation | Full       | Limited     | No         |\n| Alternative Routes  | Full       | Cldr        | Full       |\n| Translation         | ☑          | ☑          |  ☑         |\n| Route Helpers       | ☑          | ☑          |  ☑         |\n| Verified Routes     | ☑          | ☑          |  ☐         |\n| Drop-in replacement | ☑     [^3] | ☐          |  ☑         |\n| Standalone          | ☑          | ☐          |  ☐         |\n| Modular             | ☑          | ☐          |  ☐         |\n| Extendable          | ☑          | ☐          |  ☐         |\n\n[^1]: Routex' `preprocesss_using` is not bound to Phoenix (session) scopes\n[^2]: [Crazy example](https://github.com/BartOtten/routex/blob/main/lib/routex/extension/cloak.ex)\n[^3]: Routex *can* be configured to shim original Phoenix functionality (for\n    example: `~p` and `url/2`) while Cldr Routes mandates code modifications\n    (for example: `~p` -> `~q` and `url/2` -> `url_q/2`)","ref":"readme.html#comparison-table"},{"type":"extras","title":"Extensions - README","doc":"Routex relies on extensions to provide features. Each extension provides a\nsingle feature and should minimize hard dependencies on other extensions.\nInstead, Routex advises to make use of `Routex.Attrs` to share attributes;\nallowing extensions to work together whithout being coupled.\n\nThe documentation of each extension lists any provided or required\n`Routex.Attrs`.","ref":"readme.html#extensions"},{"type":"extras","title":"Alternatives - README","doc":"Create alternative routes based on `branches` configured in a Routex backend\nmodule. Branches can be nested and each branch can provide it's own attributes to\nbe shared with other extensions.\n\n[Alternatives Documentation](`Routex.Extension.Alternatives`)","ref":"readme.html#alternatives"},{"type":"extras","title":"Translations - README","doc":"This extension extracts segments of a routes' path to a `routes.po` file for\ntranslation. At compile-time it combines the translated segments to translate\nroutes. As a result, users can enter URLs using localized terms which can\nenhance user engagement and content relevance.\n\n[Translations Documentation](`Routex.Extension.Translations`)","ref":"readme.html#translations"},{"type":"extras","title":"Multilingual Routes - README","doc":"The Alternatives extension can be combined with the Translations extension to\ncreate multilingual routes. The Alternatives extension can provide the :locale\nattribute used by the Translations extension.\n\n\tOriginal            Step 1: Alternatives               Step 2: Translations\n\t\t\t\t\t\t⇒ /products/:id/edit               ⇒ /products/:id/edit\n\t/products/:id/edit  ⇒ /eu/nederland/products/:id/edit  ⇒ /eu/nederland/producten/:id/bewerken\n\t\t\t\t\t\t⇒ /eu/espana/products/:id/edit\t   ⇒ /eu/espana/producto/:id/editar\n\t\t\t\t\t\t⇒ /gb/products/:id/edit            ⇒ /gb/products/:id/edit","ref":"readme.html#multilingual-routes"},{"type":"extras","title":"Alternative Getters - README","doc":"Creates a helper function `alternatives/1` to get a list of alternative slugs\nand their route attributes.  The current route is also included and has attribute\n`current?: true`. As `Routex` sets the `@url` assign you can simply\nget all routes to the current page with `alternatives(@url)`, use an attribute in the\nroute as button text and highlight the current active route button.\n\n[Alternative Getters Documentation](`Routex.Extension.AlternativeGetters`)","ref":"readme.html#alternative-getters"},{"type":"extras","title":"Verified Routes - README","doc":"Routex is fully compatible with Verified Routes.\n\nThis extension creates a sigil (default: `~l`) with the ability to branch based\non the current alternative branch of a user. It is able to verify routes even\nwhen thy have been transformed by Routex extensions. Optionally this sigil can\nbe set to `~p` (Phoenix' default) as it is a drop-in replacement.\n\nIt also provides branching variants of `url/{2,3,4}` and `path/{2,3}`.\n\n[Verified Routes Documentation](`Routex.Extension.VerifiedRoutes`)","ref":"readme.html#verified-routes"},{"type":"extras","title":"Route Helpers - README","doc":"Creates Phoenix Helpers that have the ability to branch based on the current\nalternative branch of a user. Optionally these helpers can replace the original\nPhoenix Route Helpers as they are drop-ins.\n\n[Route Helpers Documentation](`Routex.Extension.RouteHelpers`)","ref":"readme.html#route-helpers"},{"type":"extras","title":"Interpolation - README","doc":"With this extension enabled, *any* attribute assigned to a route can be used\nfor route interpolation. Most effective with an extension which enables\nalternative routes generation (such as extension `Alternatives`).\n\n    /product/#{territory}/:id/#{language}  => /product/europe/:id/nl\n\n[Interpolation Documentation](`Routex.Extension.Interpolation`)","ref":"readme.html#interpolation"},{"type":"extras","title":"Assigns - README","doc":"With this extension you can add (a subset of) attributes set by other extensions\nto Phoenix' assigns making them available in components and controllers with the\n`@` assigns operator (optionally under a namespace)\n\n    @namespace.area      =>  :eu_nl\n\t@namespace.contact   =>  \"contact@example.com\"\n\n[Assigns Documentation](`Routex.Extension.Assigns`)","ref":"readme.html#assigns"},{"type":"extras","title":"Attribute Getters - README","doc":"Creates a helper function `attrs/1` to get all `Routex.Attrs` of a route. As\n`Routex` sets the `@url` assign you can simply get all attributes for the\ncurrent page with `attrs(@url)`.\n\nThis way the `assigns` can be a subset of the full list of attributes but the\nfull list can be lazy loaded when needed.\n\n[Attribute Getters Documentation](`Routex.Extension.AttrGetters`)","ref":"readme.html#attribute-getters"},{"type":"extras","title":"Cldr Adaptor - README","doc":"Adapter for projects using :ex_cldr.\n\n[Cldr Adaptor Documentation](`Routex.Extension.Cldr`)","ref":"readme.html#cldr-adaptor"},{"type":"extras","title":"Cloak (show case) - README","doc":"Transforms routes to be unrecognizable. This extension is a show case and may\nchange at any given moment to generate other routes without prior notice.\n\nIn this example it numbers all routes starting at 01 and increments the counter\nfor each route. It also shifts the parameter to the left; causing a chaotic\nroute structure. Do note: this still works with the Verified Routes extension\nwhile using the standard route (e.g. `<.link navigate={~p\"/products\">`) in\ntemplates.\n\n\n      Original                 Rewritten     Result (product_id: 88, 89, 90)\n      /products                ⇒ /01         ⇒   /01\n      /products/:id/edit       ⇒ /:id/02     ⇒ /88/02, /89/02, /90/02 etc...\n      /products/:id/show/edit  ⇒ /:id/03     ⇒ /88/03, /89/03, /90/03 etc...\n\n\n[Cloak Documentation](`Routex.Extension.Cloak`)","ref":"readme.html#cloak-show-case"},{"type":"extras","title":"Usage","doc":"# Usage","ref":"usage.html"},{"type":"extras","title":"Requirements - Usage","doc":"- Elixir >=1.11\n- Phoenix >= 1.6.0\n- Phoenix LiveView >= 0.16 (optional)","ref":"usage.html#requirements"},{"type":"extras","title":"Installation - Usage","doc":"You can install this library by adding it to your list of dependencies in `mix.exs`. (use `mix hex.info routex` to find the latest version):\n\n```diff\ndef deps do\n  [\n     ...other deps\n+    {:routex, \">= 0.0.0\"}\n  ]\nend\n```\n\nModify the entrypoint your web interface definition.\n```diff\n# file: lib/example_web.ex\n\n+  use Routex.Router  # always before Phoenix Router\n   use Phoenix.Router, helpers: true\n\n# in controller\n   unquote(verified_routes())\n+  unquote(routex_helpers())\n\n# in live_view\n      unquote(html_helpers())\n+     on_mount(unquote(__MODULE__).Router.RoutexHelpers)\n\n# in view_helpers or html_helpers\n   unquote(verified_routes())\n+  unquote(routex_helpers())\n\n# insert new private function\n+  defp routex_helpers do\n+    quote do\n+      import unquote(__MODULE__).Router.RoutexHelpers, only: :macros\n+      alias unquote(__MODULE__).Router.RoutexHelpers, as: Routes\n+    end\n+  end\n```\n\nThe `on_mount` hook attaches a `handle_param` which in turn assigns a few\n(helper) values to the connection and/or socket. This includes the current url\nand any assigns created by Routex Extensions. When you want to have full control\nover these hooks, you can use something like the snippet below instead.\n\n```elixir\ndef on_mount(_, params, session, socket) do\n  socket =\n    Phoenix.LiveView.attach_hook(socket, :set_rtx, :handle_params, fn _params, url, socket ->\n      attrs = ExampleWeb.Route.RoutexHelpers.attrs(url)\n      rtx_assigns = [url: url, __branch__: attrs.__branch__] ++ Map.to_list(attrs.assigns)\n\n      {:cont, Phoenix.LiveView.assign(socket, rtx_assigns)}\n    end)\n\n  {:cont, socket}\nend\n```","ref":"usage.html#installation"},{"type":"extras","title":"Configuration - Usage","doc":"To use `Routex`, a module that calls `use Routex.Backend` (referred to below as a\n\"backend\") has to be defined. It includes a list with extensions and\nconfiguration of extensions.\n\n```elixir\ndefmodule ExampleWeb.RoutexBackend do\nuse Routex.Backend,\n  extensions: [\n  # ...list of extensions...\n  ],\nend\n```","ref":"usage.html#configuration"},{"type":"extras","title":"Extensions - Usage","doc":"Routex is merely a framework and relies on extensions to provide features. Each\nextension provides a single feature. The extensions have their own documentation\nwhich specifies how to configure and use them.","ref":"usage.html#extensions"},{"type":"extras","title":"Preprocess routes with Routex - Usage","doc":"`Routex` will preprocess any route wrapped in a `preprocess_using` block; either\ndirect or indirect. It uses the backend passed as the first argument.  This\nallows the use of multiple backends (e.g. to use different extensions for admin\nroutes)\n\n```diff\n# file: router.ex\n  scope \"/\", ExampleWeb, host: \"admin.\", as: :admin do\n    pipe_through :browser\n\n+    preprocess_using ExampleWeb.RoutexBackendAdmin do\n      # [...routes...]\n+    end\n  end\n\n+ preprocess_using ExampleWeb.RoutexBackend do\n    scope \"/\", ExampleWeb do\n      pipe_through [:browser, :redirect_if_user_is_authenticated]\n      # [...routes...]\n    end\n\n    scope \"/\", ExampleWeb do\n      pipe_through [:browser, :require_authenticated_user]\n      # [...routes...]\n    end\n+ end\n```\n\nWhen you run into issues, please have a look at the [Troubleshooting](TROUBLESHOOTING.md)","ref":"usage.html#preprocess-routes-with-routex"},{"type":"extras","title":"Change Log","doc":"# Change Log\n\nAll notable changes to this project will be documented in this file.\nSee [Conventional Commits](Https://conventionalcommits.org) for commit guidelines.\n\n<!-- changelog -->\n\n## [v0.3.0-alpha.3](https://github.com/BartOtten/routex/compare/v0.3.0-alpha.2...v0.3.0-alpha.3) (2024-10-28)","ref":"changelog.html"},{"type":"extras","title":"Features: - Change Log","doc":"* cldr: add adaptor for projects using Cldr","ref":"changelog.html#features"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"* core: preprocess_using does not accept options\n\n* mix: do not override dep :ssl_verify_fun\n\n## [v0.3.0-alpha.2](https://github.com/BartOtten/routex/compare/v0.3.0-alpha.1...v0.3.0-alpha.2) (2024-10-23)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Breaking Changes: - Change Log","doc":"* dev: rename development aid functions\n\n* alternatives: rename :is_current? to :match?\n\n* translations: depend on Gettext greater than 0.26.0","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"* docs: warnings about referencing Phoenix.Router.Route\n\n* core: rename is_private/1 to private?/1\n\n* ci: update Github actions\n\n* core: warning during compilation about usage of :warn\n\n## [v0.3.0-alpha.1](https://github.com/BartOtten/routex/compare/v0.3.0-alpha.1...v0.3.0-alpha.1) (2024-10-21)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Breaking Changes: - Change Log","doc":"* core: remove Routex.Path module\n\n* core: split Extension Utils module\n\n* alternatives: rename scope to branch","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Features: - Change Log","doc":"* core: merge private Routex attrs into socket\n\n* verified: branching macro's of all arities of ~p, url and path\n\n* core: enable AST-free manipulation of routes with interpolation\n\n* alternatives: indicate if an alternative route `is_current?`\n\n* interpol: add extension for interpolation of routes\n\n* translations: distinct locale and language\n\n* core: introduce Branching module\n\n* core: introduce Matchable module\n\n## [v0.2.0-alpha.8](https://github.com/BartOtten/routex/compare/v0.2.0-alpha.7...v0.2.0-alpha.8) (2023-10-18)","ref":"changelog.html#features"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"* translations: reliably extract segments to translate\n\n## [v0.2.0-alpha.7](https://github.com/BartOtten/routex/compare/v0.1.0-alpha.7...v0.2.0-alpha.7) (2023-05-31)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Features: - Change Log","doc":"* utils: use verbose defaults for esc_inspect/2","ref":"changelog.html#features"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"* route: swallowed routes due to nonunique map key\n\n## [v0.1.0-alpha.6](https://github.com/BartOtten/routex/compare/v0.1.0-alpha.5...v0.1.0-alpha.6) (2023-05-30)\n\n\n\n\n## [v0.1.0-alpha.5](https://github.com/BartOtten/routex/compare/v0.1.0-alpha.5...v0.1.0-alpha.5) (2023-05-05)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Features: - Change Log","doc":"* add ROUTEX_DEBUG for compilation debugging (#3)","ref":"changelog.html#features"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"* verified: let Phoenix handle missing routes","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Routex Extensions","doc":"# Routex Extensions\n\n> #### List of extensions {: .info}\n> A [list of included extensions](README.md#extensions) can be found in the README.\n\nRoutex Extensions extend the functionality provided by Routex to transform\nroutes or generate new route based helper functions. Each extension is a module\nwhich implements the `Routex.Extension` behaviour. It has to implement one or\nmultiple callbacks:\n\n- configure/2\n- transform/3\n- post_transform/3\n- create_helpers/3\n\nRoutex will call those callbacks at different stages before Routex handsoff the\nresult to `Phoenix.Router` for compilation.","ref":"extensions.html"},{"type":"extras","title":"Callbacks and stages - Routex Extensions","doc":"","ref":"extensions.html#callbacks-and-stages"},{"type":"extras","title":"Stage 1: Configure - Routex Extensions","doc":"This stage enables extensions to pre-process backend options upfront.\n\nThe `configure/2` callback is called with the options provided to\n`Routex.Backend` and the name of the Routext backend. It is expected to return a\nnew list of options.\n\nRoutex collects all options in this stage for subsequent stages. Therefore,\nextensions should add any fallback/default they might use themselves to the\noptions in this stage.\n\nTo aid in code completion, the final configuration is passed as a struct to\nsubsequent stages.","ref":"extensions.html#stage-1-configure"},{"type":"extras","title":"Stage 2: Transform - Routex Extensions","doc":"This stage is meant to change the properties of routes, which are at that moment\n`Phoenix.Router.Route` structs. The routes are grouped by Routext backend and\nprocessed per group, allowing an extension to use accumulating values within one\niteration.\n\nThe `transform/3` callback is called with a list of routes belonging to a\nRoutext backend, the name of the backend and the current environment. It is\nexpected to return a list of Phoenix.Router.Route structs.\n\n#### Flattening option values\nExtensions can make use of `Routex.Attrs` provided by Routex itself, Routex\nbackends and other extensions.\n\nTo make the availability of the attributes as predictable as possible, Routex\nuses a flat structure which is stored in a routes' `private.routex` field.\nHowever, using a flat structure might conflict with developer experience;\nsometimes a nested structure to provide configuration options might be more\nsuitable.\n\nOne responsibility of the `transform/3` callback is to flatten the structure of\nattributes they use for each route they receive, so other extensions can use\nattributes set by the current extension without knowledge of the configuration\nstructure.\n\n**Example**\nThe Alternatives extension uses nested options and allows inheritance\nof properties from parent branches.\n\n```\nalternatives: %{\n  \"/\" =>\n    helper: nil,\n    locale: \"en\",\n    branches: %{\n      \"nl\" => %{\n          helper: \"nl\",\n          locale: \"nl\"\n        },\n      \"gb\" => %{\n        helper: \"gb\",\n        }\n    }\n}\n```\nThe Alternatives module is therefor responsible for flattening those for\n(itself and) other extensions to use. To take the route responsible for the\n\"gb\" branch as an example, the extension should add flattened attributes in the\nRoute struct. It can do so using the helper function `Routex.Attrs.put/2`.\n\n```\nRoutex.Attrs.put(route, [locale: \"en_GB\", helper: \"gb\"])\n```\n\nNow the `Translation` extension can search for the option `:locale` in the\nroute's opts, unaware of how that locale was initially configured.","ref":"extensions.html#stage-2-transform"},{"type":"extras","title":"Stage 3: Post Transform - Routex Extensions","doc":"The `post_transform` stage is meant to set `Routex.Attrs` knowing all other\nproperties of a route are available.","ref":"extensions.html#stage-3-post-transform"},{"type":"extras","title":"Stage 4: Create helpers - Routex Extensions","doc":"In this stage helper functions can be generated which will be added to\n`MyAppWeb.Router.RoutexHelpers`.\n\nThe `create_helpers/3` callback is called with a list of routes belonging to a\nRoutext backend, the name of the Routext backend and the current environment.\nIt is expected to return Elixir AST.\n\nAs a result the developer only has to `import MyAppWeb.Router.RoutexHelpers`\nfor all helpers generated by extensions to be included in the app.","ref":"extensions.html#stage-4-create-helpers"},{"type":"extras","title":"Guidelines - Routex Extensions","doc":"* make functions not defined by the `Routex.Extension` behaviour private.\n* provide as many options as possible; other extensions might use the information.\n* provide additional options as flat list(s) so other extensions don't have to guess structure.\n* as other extensions might use options set by your extension, try to preserve any previously defined option.","ref":"extensions.html#guidelines"},{"type":"extras","title":"Documentation - Routex Extensions","doc":"@moduledoc \"\"\"\n    Summary of feature provided.","ref":"extensions.html#documentation"},{"type":"extras","title":"Options - Routex Extensions","doc":"- `name` - description","ref":"extensions.html#options"},{"type":"extras","title":"Example configuration - Routex Extensions","doc":"```diff\n    # file /lib/example_web/routex_backend.ex\n    defmodule ExampleWeb.RoutexBackend do\n      use Routex.Backend,\n      extensions: [\n    +   Routex.Extension.Name\n        Routex.Extension.Attrs\n    ],\n    + name_config: [name_opt: \"value\"]\n    ```","ref":"extensions.html#example-configuration"},{"type":"extras","title":"Pseudo result - Routex Extensions","doc":"/products/:id/edit  ⇒ /products/:id/edit\n\n    ## `Routex.Attrs`\n    **Requires**\n    - none\n\n    **Sets**\n    - none","ref":"extensions.html#pseudo-result"},{"type":"extras","title":"Helpers - Routex Extensions","doc":"function_name(arg1 :: type) :: type\n    \"\"\"","ref":"extensions.html#helpers"},{"type":"extras","title":"Troubleshooting","doc":"# Troubleshooting","ref":"troubleshooting.html"},{"type":"extras","title":"Compilation - Troubleshooting","doc":"When your application fails to compile you might find the cause by setting the\nenvironment variable `ROUTEX_DEBUG` to `true`.\n\n    ROUTEX_DEBUG=true mix compile\n\nDo note that this might show early compilation issues, but will make the final\ncompilation fail at all times.","ref":"troubleshooting.html#compilation"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}